#!/usr/bin/env python
# -*- coding: utf-8 -*-
import os
import io
import sys
import pkgutil

import xin2pbn.xin2pbn as xin2pbn
import pbn2html.pbn2html as pbn2html

from .lin2pbn import bbo2pbn

import requests
import re

from string import Template

def wrapper(str):
    #print(str)
    wrapper="""
```{=latex}
%s
```
""" % str.strip()
    return wrapper

auction_template=r"""\begin{quote}
	\begin{bidding}
		$auction
	\end{bidding}
\end{quote}
"""

board_template=r"""\begin{quote}
	\crdima
    {\begin{minipage}[t]{\br}$ul\end{minipage}}
    {\begin{minipage}[t]{\br}$ll\end{minipage}}
    {$north}
    {$west}
    {$east}
    {$south}
    {\begin{minipage}[t]{\br}$ur\end{minipage}}
\end{quote}
"""

pbn={}

def hands_parse(deal):
    # deal=.xxxx..xxx&.94.A.AKT7&-&.AKQ6.865.
    # hands = {
    #    'W': {'S': '8 2', 'H': 'J 10 8', 'D': '10 9 7 4', 'C': 'J 10 5 3'}, 
    #    'N': {'S': 'x J 3', 'H': '9 4', 'D': 'A J', 'C': 'A K Q 8 7 4'}, 
    #    'E': {'S': '10 9 6 4', 'H': '7 5 3 2', 'D': 'K Q 2', 'C': '9 6'}, 
    #    'S': {'S': '无 Q 7 5','H': 'A K Q 6', 'D': '8 6 5 3', 'C': '2'}
    #}
    allhands = deal.split("&")
    wnes = []
    for hand in allhands:
        shdc = []
        if hand == "-":
            shdc = ["S","K","I","P"]
        else:
            shdc = [" ".join(i) for i in hand.split(".")]
            shdc = [x.replace('T', '10') for x in shdc]
            shdc = [x if x != "" else "—" for x in shdc]

        zip_iterator = zip(['S','H','D','C'], shdc)
        wnes.append(dict(zip_iterator))
        #print(wnes)
    zip_iterator = zip(['W','N','E','S'], wnes)
    hands = dict(zip_iterator)
    return hands #"\hand{}{94}{A}{AK87}"

def bidding_parse(auction):
    return(auction)

def latex_suit(suit):
    # c => 
    for word, initial in {"S":"♠", "H":"♥","D":"♦","C":"♣" }.items():
        suit = suit.replace(word, initial)
    return suit

def pbn_latex_auction(pbn):
    src = Template(auction_template)
    # Pass \> 2NT\> Pass \> 3\c\\
    # Pass \> 3NT\> Pass \> 6NT\\
    # All Pass
    # need insert empty cell based on auction
    auction = pbn["tags"]["Auction"]
    section_auction = pbn["section_auction"]

    position="WNES" 
    empty_cells = position.index(auction)

    # handle ==1==, 6D?, 6D!
    filtered_auction = [x for x in section_auction.split() if not x.startswith("=")]
    # print(filtered_auction)
    table = ""
    col = 0
    for empty in range(empty_cells):
        table += " \\>"
        col += 1
    for one in filtered_auction:
        # print("aution:", one)
        one = one.replace("!", '')
        table +="%s " % latex_suit(one)
        if col == 3:
            col = 0
            table += "\\\\\n"
        else:
            col += 1
            table += "\\>"
    all = { "auction": table}
    return src.safe_substitute(all)

def latex_info(info,bottom=False):
    if bottom:
        # easiest way to put into bottom with extra newline &, totally for 4 lines
        for i in range(3 - info.count('&')):
            info="\hfill\\\\" + info
    info = info.replace("&","\\\\")  # line break with \\
        
    return info

def latex_card(cards):
    # 'W': {'S': '8 2', 'H': 'J 10 8', 'D': '10 9 7 4', 'C': 'J 10 5 3'}, 
    str = "\hand{%s}{%s}{%s}{%s}" % (cards["S"].replace(" ", ""), cards["H"].replace(" ", ""),cards["D"].replace(" ", ""),cards["C"].replace(" ", ""))
    return str #"\hand{}{94}{A}{AK87}"

def pbn_latex_deal(pbn, cards="NESW", ll="", ul="", ur=""):
    all = {}
    tags = pbn["tags"]
    hands = pbn["hands"]
    empty_hand = "\\nonhand{ }{ }{ }{ }"
    # print(hands["N"])

    all["north"] = latex_card(hands["N"])
    all["west"] = latex_card(hands["W"])
    all["east"] = latex_card(hands["E"])
    all["south"] = latex_card(hands["S"])
    for card in "NESW":
        if card not in cards:
            if card == "N":
                # all["north"] = empty_hand
                # use space can hide the cell
                all["north"] = ""
            if card == "W":
                all["west"] = empty_hand
            if card == "E":
                all["east"] = empty_hand
            if card == "S":
                #all["south"] = empty_hand
                # use space can hide the cell
                all["south"] = ""

    if ul == "":
        all["ul"] = latex_info("Dealer: " + tags["Dealer"] + "\\\\" + "Vul: " + tags["Vulnerable"])
    else:
        all["ul"] = latex_info(ul)
    all["ll"] = latex_info(ll, bottom=True)
    all["ur"] = latex_info(ur)

    src = Template(board_template)
    #template = open("deal_template.html", "r", encoding="utf-8").read()
    #src = Template(template
    # print(all)
    result = src.safe_substitute(all)
    return result

def convert(block):
    global pbn
    PBN_FILE="interesting"
    result = ""
    for line in block:
        #print(line)
        if line.startswith("http"):
            # print("heelo:", line)
            if "DealLog" in line: # xinrui
                xin2pbn.xin2pbn(line, PBN_FILE)
                pbn = pbn2html.get_from_pbn_file(PBN_FILE+".pbn")
                # print(pbn)
            elif "bridgebase" in line: # bbo lin format
                bbo2pbn(line, PBN_FILE+".pbn")
                pbn=pbn2html.get_from_pbn_file(PBN_FILE+".pbn")
            else:
                continue
            # print(pbn)
        elif line.startswith("auction"):
            # handle part later to change pbn
            newpbn=pbn.copy()
            kv = line.split("=")
            if len(kv) > 1:
                auction=kv[1]
                #print(newpbn["hands"])
                newpbn["section_auction"] = bidding_parse(auction)
            result += wrapper(pbn_latex_auction(newpbn))
        elif line.startswith("deal"):
            options = line.strip().split("|")
            cards = "NEWS"
            ll = ul = ur = ""
            newpbn=pbn.copy()
            for option in options:
                if option.startswith("deal"):
                    # handle part later to change pbn
                    kv = option.split("=")
                    if len(kv) > 1:
                        deal=kv[1]
                        #print(newpbn["hands"])
                        newpbn["hands"] = hands_parse(deal)
                else:
                    key,value = option.split("=")
                    if key=="cards":
                        cards = value
                    elif key == "ll": # lower left
                        ll = value.strip('\"')
                    elif key == "ul":
                        ul = value.strip('\"')
                    elif key == "ur":
                        ur = value.strip('\"')
            result += wrapper(pbn_latex_deal(newpbn, cards=cards, ll=ll,ul=ul, ur=ur))
    return result

def process_md(md_file):
    output = os.path.splitext(md_file)[0]+'.bridge-tex'
    print("processing %s -> %s" % (md_file, output))

    with io.open(output, "w", encoding="utf-8") as md_target:
        with io.open(md_file, encoding="utf-8") as md_orig:
            block = []
            found = False

            for line in md_orig.readlines():
                if found:
                    if line.strip() == "</pre>":
                        #print("come here", block)
                        result = convert(block)
                        md_target.write(result)
                        found = False
                    else:
                        block.append(line)
                else:
                    if line.rstrip() == '<pre lang="bridge">':
                        found = True
                        block = []
                    else:
                        md_target.write(line)

def main():
    for md in sys.argv[1:]:
        process_md(md)

if __name__ == '__main__':
    main()


"""
Code for parsing PBN files borrowed from alorenzen/DoubleDummy
https://github.com/alorenzen/DoubleDummy/blob/master/pbn_parser.py
which comes from
http://4coder.org/python-source-code/18/pybridge-0.3.0/pybridge/bridge/pbn.py.html
"""
import os
from .game_state import *
#from game_state import *


class ParseError(Exception):
    """Raised when PBN parser encounters an unexpected input."""

def importPBN(pbn):
    SUIT = 'SHDC'
    CARDRANK = dict(zip('23456789TJQKA',range(2,15)))
    POSITION = dict(zip('NESW',Player.POSITION))
    PLAYERINDEX = dict(zip('NESW',range(1,5)))
    tags, sections, notes = parsePBN(pbn)
    # try to fetch lead
    if "Play" in sections:
        tags["Lead"] = sections["Play"].split()[0]
    else:
        tags["Lead"] = ""
    if "Event" not in tags:
        tags["Event"] = ""
    #print(tags,sections, notes)
    if 'Deal' not in tags:
        raise ParseError("Required tag 'Deal' not found")
    first, cards = tags['Deal'].split(":")
    index = PLAYERINDEX[first.strip()] - 1
    #print(PLAYERINDEX)
    #print(POSITION)
    #print(Player.POSITION)
    order = Player.POSITION[index:] + Player.POSITION[:index]
    # print(first,cards,index, order)
    hands = {}
    for player, hand in zip(order, cards.strip().split()):
        # print("loop:", player,hand)
        hands[player] = {}
        for suit, suitcards in zip(SUIT, hand.split('.')):
            hands[player][suit] = ""
            for rank in suitcards:
                card = Card(suit,CARDRANK[rank])
                if hands[player][suit] != "":
                    hands[player][suit] += " "
                if rank == "T":
                    rank ="10"
                hands[player][suit] += rank
            if hands[player][suit] == "":
                hands[player][suit] = "—"
    #print(hands)
    #print((sections['Optimumresulttable'].split('\n'))[1:20])
    #results = filter(lambda x: x[1] == 'NT',map(lambda x: x.strip().split(),sections['Optimumresulttable'].split('\n'))[1:20])
    #expected = {}
    #for player, suit, tricks in results:
    #    player = POSITION[player]
    #    deal = Deal(hands,player)
    #    if not deal.isValidDeal():
    #        raise ParseError("Deal does not validate")
    #    expected[player] = (deal,tricks)
    return tags, hands, sections["Auction"]

def parsePBN(pbn):
    """Parses the given PBN string and extracts:
    
      * for each PBN tag, a dict of associated key/value pairs.
      * for each data section, a dict of key/data pairs.
        
    This method does not interpret the PBN string itself.
    
    @param pbn: a string containing PBN markup.
    @return: a tuple (tag values, section data, notes).
    """
    tagValues, sectionData, notes = {}, {}, {}
    tag = 'Default Tag'
    for line in pbn.splitlines():
        line.strip()  # Remove whitespace.

        if line.startswith('%'):  # A comment.
            pass  # Skip over comments.

        elif line.startswith('['):  # A tag.
            line = line.strip('[]')  # Remove leading [ and trailing ].
            # The key is the first word, the value is everything after.
            tag, value = line.split(' ', 1)
            tag = tag.capitalize()
            value = value.strip('\'\"')
            if tag == 'Note':
                notes.setdefault(tag, [])
                notes[tag].append(value)
            else:
                tagValues[tag] = value

        else:  # Line follows tag, add line to data buffer for section.
            sectionData.setdefault(tag, '')
            sectionData[tag] += line + '\n'

    return tagValues, sectionData, notes


def testImport():
    pbnstr = file("pbn_files/OptimumResultTable.pbn").read()
    g = importPBN(pbnstr)
    return g



#!/usr/bin/env python
# -*- coding: utf-8 -*-
import os
import io
import sys
import pkgutil

import requests
import re

from string import Template
from collections import deque

def numToDirection(n):
    return ['S', 'W', 'N', 'E'][n%4]

# this code copied from https://github.com/kevinywlui/lin2pbn/blob/master/lin2pbn.py
# http://www.himbuv.com/nieuw/english/lin_en
# https://github.com/morgoth/lin/blob/master/lib/lin/parser.rb
def lin2pbn(linstring, pbnfile):
    linList = linstring.split('||')

    # Extract Players SWNE
    [south, west, north, east] = linList[0].split('|')[1].split(',')

    # Extract hands
    dd = linList[1].split('|')[1].split(',')
    hands = [list(map(lambda x: x[::-1], re.split('S|H|D|C', d))) for d in dd]
    [dealerIndex, _, _, _] = [x.pop(0) for x in hands]
    
    dealerIndex = int(dealerIndex)
    dealer = numToDirection(dealerIndex -1)
    if len(hands[3]) == 0:
        rank = 'AKQJT98765432'
        hands[3] = 4 * ['']
        for i in range(4):
            for j in range(13):
                if all([rank[j] not in hands[k][i] for k in range(3)]):
                    hands[3][i] = hands[3][i] + rank[j]
    # hands always starts from S now, so need rotate
    # hands.rotate(dealerIndex -1)
    for _ in range(dealerIndex -1):
        hands.append(hands.pop(0))
    # print(hands)
    deal = dealer + ":" + " ".join([".".join(x) for x in hands])

    # Extract auction
    aa = linList[2].split('|')[1::2]
    board = aa[0].split(' ')[1]
    if aa[1] == 'o':
        vulnerable = 'None'
    elif aa[1] == 'e':
        vulnerable = 'ES'
    elif aa[1] == 'n':
        vulnerable = 'NS'
    else:
        vulnerable = 'All'

    bids = aa[2::]
    bids = ['Pass' if x == 'p' else x for x in bids]
    auction = ''
    for i in range(len(bids) // 4):
        t = ['', '', '', '']
        for j in range(4):
            if 4 * i + j < len(bids):
                t[j] = bids[4 * i + j]
        auction = auction + '{0[0]:<7}{0[1]:<7}{0[2]:<7}{0[3]:<7}\n'.format(t)

    stake = ''
    for x in bids[::-1]:
        if x == 'XX':
            stake = 'XX'
            continue
        if x == 'X' and stake != 'XX':
            stake = 'X'
            continue
        if x[1] in ['S', 'H', 'C', 'D', 'N']:
            contract = x + stake
            strain = x[1:]
            break

    for x in bids:
        if x[1:] == strain:
            declarerIndex = bids.index(x)
            break
    declarer = ['S', 'W', 'N', 'E'][(declarerIndex+dealerIndex - 1) % 4]

    # Extract play
    pp = linList[3::]
    pp = [p.split('|')[1::2] for p in pp]
    if pp[-1] == []:
        claim = 0
    else:
        claim = pp[-1][-1] 
    
    play = ''
    for x in pp:
        if x == []:
            continue
        t = ['', '', '', '']
        for j in range(len(x)):
            t[j] = x[j]
        play = play + '{0[0]:<4}{0[1]:<4}{0[2]:<4}{0[3]:<4}\n'.format(t)
    play = [numToDirection(declarerIndex+1),play]

    PBNString = ''
    PBNString = PBNString + '[Event \"\"]\n'
    PBNString = PBNString + '[Site \"\"]\n'
    PBNString = PBNString + '[Date \"\"]\n'
    PBNString = PBNString + '[Board \"{}\"]\n'.format(board)
    PBNString = PBNString + '[West \"{}\"]\n'.format(west)
    PBNString = PBNString + '[North \"{}\"]\n'.format(north)
    PBNString = PBNString + '[East \"{}\"]\n'.format(east)
    PBNString = PBNString + '[South \"{}\"]\n'.format(south)
    PBNString = PBNString + '[Dealer \"{}\"]\n'.format(dealer)
    PBNString = PBNString + '[Vulnerable \"{}\"]\n'.format(vulnerable)
    PBNString = PBNString + '[Deal \"{}\"]\n'.format(deal)
    PBNString = PBNString + '[Scoring \"\"]\n'
    PBNString = PBNString + '[Declarer \"{}\"]\n'.format(declarer)
    PBNString = PBNString + '[Contract \"{}\"]\n'.format(contract)
    PBNString = PBNString + '[Result \"\"]\n'
    PBNString = PBNString + '[Auction \"{}\"]\n{}'.format(dealer, auction)
    PBNString = PBNString + '[Play \"{0[0]}\"]\n{0[1]}'.format(play)
    with io.open(pbnfile, "w", encoding="utf-8") as text_file:
        print("write to file %s" % pbnfile)
        text_file.write(PBNString)

def bbo2pbn(url, pbnfile):
        # URL of the image to be downloaded is defined as image_url 
    r = requests.get(url) # create HTTP response object 
    lin_str = r.text
    print("Download from %s and lin is:\n%s" % (url, lin_str))
    lin2pbn(lin_str, pbnfile)

def main():
#  https://www.bridgebase.com/myhands/fetchlin.php?id=775620181&when_played=1610708401
    # pn|precisionb,Que10plm,Jansty,Rollo100|st||md|2S7TJH289JQAD2C59A,S9H7D489JQKC3468Q,S248QAH5KD36TC7JK,|rh||ah|Board 4|sv|b|mb|p|mb|1S|mb|p|mb|2H|mb|p|mb|p|mb|p|pg||pc|DK|pc|D3|pc|D5|pc|D2|pg||pc|DQ|pc|D6|pc|D7|pc|H2|pg||pc|H8|pc|H7|pc|HK|pc|H3|pg||pc|H5|pc|H4|pc|HJ|pc|D4|pg||pc|HQ|pc|S9|pc|C7|pc|H6|pg||pc|HA|pc|C3|pc|DT|pc|HT|pg||pc|SJ|pc|D8|pc|S2|pc|SK|pg||pc|DA|pc|H9|pc|D9|pc|S4|pg||pc|CA|pc|C4|pc|CJ|pc|C2|pg||pc|ST|pc|C6|pc|S8|pc|S3|pg||mc|11|
    # |md|2S7TJH289JQAD2C59A,S9H7D489JQKC3468Q,S248QAH5KD36TC7JK,
    # [Deal "W:JT7.AQJ982.2.A95 9.7.KQJ984.Q8643 AQ842.K5.T63.KJ7 K653.T643.A75.T2"]
    # [Deal "W:9.7.KQJ984.Q8643 AQ842.K5.T63.KJ7 K653.T643.A75.T2 JT7.AQJ982.2.A95"]
    if len(sys.argv) > 1:
        url = sys.argv[1]
        pbnfile = "interesting.pbn"
        if "bridgebase" in url:
            bbo2pbn(url, pbnfile)
        else:
            lin2pbn(lin_str, pbnfile)
    else:
        print("lin2pbn linstr|bbo")

if __name__ == '__main__':
    main()


#!/usr/bin/env python
# -*- coding: utf-8 -*-
import os
import io
import sys
import pkgutil

import pkg_resources  # part of setuptools
version = pkg_resources.require("bridge-utils")[0].version

from .pbn_parser import importPBN, ParseError
#from pbn_parser import importPBN, ParseError

from string import Template

language="Chinese"

html_template="""
<!DOCTYPE html>
<html>
  <head>
    <meta charset="utf-8">
    <meta name="Generator" content="pbn2html toolkit">
    <title>Enjoy the bridge game</title>
  </head>
  <h2>Introduction</h2>
  <p>this is designed to be used (copy/paste) for HTML/Microsoft Word</p>
  <p>Designed by Larry Cai and produced by <a href="https://pypi.org/project/bridge-utils/">bridge-utils/pbn2html</a>, version $version</p>
  <p>Credit to <a href="https://bridgecomposer.com/">BridgeComposer Version 5.83 (64 bit)</a></p>
  <body class=bcbody>
    <!-- Credit to BridgeComposer Version 5.83 (64 bit) - https://bridgecomposer.com/ -->
    <div class=bcboard style="text-align: center">
        $all
  </body>
</html>
"""
card_template="""
              <table class=bchand style="border-collapse: collapse; border-spacing: 0">
                <tr class=bchand>
                  <td class=bcpip style="font: 10pt Verdana, sans-serif; padding: 1px; padding-top: 0; padding-bottom: 0; width: 1em; text-align: center"><span class=bcspades style="color: black">&spades;</span></td>
                  <td class=bchand style="font: 10pt Verdana, sans-serif; padding: 1px; padding-top: 0; padding-bottom: 0">$spade</td>
                </tr>
                <tr class=bchand>
                  <td class=bcpip style="font: 10pt Verdana, sans-serif; padding: 1px; padding-top: 0; padding-bottom: 0; width: 1em; text-align: center"><span class=bchearts style="color: red">&hearts;</span></td>
                  <td class=bchand style="font: 10pt Verdana, sans-serif; padding: 1px; padding-top: 0; padding-bottom: 0">$heart</td>
                </tr>
                <tr class=bchand>
                  <td class=bcpip style="font: 10pt Verdana, sans-serif; padding: 1px; padding-top: 0; padding-bottom: 0; width: 1em; text-align: center"><span class=bcdiams style="color: red">&diams;</span></td>
                  <td class=bchand style="font: 10pt Verdana, sans-serif; padding: 1px; padding-top: 0; padding-bottom: 0">$diamond</td>
                </tr>
                <tr class=bchand>
                  <td class=bcpip style="font: 10pt Verdana, sans-serif; padding: 1px; padding-top: 0; padding-bottom: 0; width: 1em; text-align: center"><span class=bcclubs style="color: black">&clubs;</span></td>
                  <td class=bchand style="font: 10pt Verdana, sans-serif; padding: 1px; padding-top: 0; padding-bottom: 0">$club</td>
                </tr>
              </table>
            </td>
"""

card_empty_template="""
    <table class=bchand style="border-collapse: collapse; border-spacing: 0">
    </table>
"""

board_template="""
              <table class=bcct style="width: 4em; height: 4em; border-collapse: collapse; font: 10pt simsun, sans-serif; color: #ffffff; border: 1px solid #aaaaaa">
                <tr class=bcct1>
                  <td class=bcct1 style="text-align: center; vertical-align: text-top; padding: 1px"></td>
                  <td class=bcct1 style="text-align: center; vertical-align: text-top; padding: 1px;background-color: $ns_vulnerable;">N</td>
                  <td class=bcct1 style="text-align: center; vertical-align: text-top; padding: 1px"></td>
                </tr>
                <tr class=bcct2>
                  <td class=bcct2a style="text-align: left; vertical-align: text-top; padding: 1px; background-color: $ew_vulnerable;">W</td>
                  <td class=bcct2b style="text-align: center; padding: 1px; color: black;">$dealer</td>
                  <td class=bcct2c style="text-align: right; vertical-align: text-top; padding: 1px; background-color: $ew_vulnerable;">E</td>
                </tr>
                <tr class=bcct3>
                  <td class=bcct3 style="text-align: center; vertical-align: text-top; padding: 1px"></td>
                  <td class=bcct3 style="text-align: center; vertical-align: text-top; padding: 1px; background-color: $ns_vulnerable;">S</td>
                  <td class=bcct3 style="text-align: center; vertical-align: text-top; padding: 1px"></td>
                </tr>
              </table>
"""
extra_template="""
            <td class=bchd3 style="padding: 1px; font: 10pt Verdana, sans-serif; padding: 1px; text-align: left; vertical-align: top;">
                $contract by $declarer<br />Lead: $lead
            </td>
"""

info_template="""
            <td class=bchd3 style="padding: 1px; font: 10pt Verdana, sans-serif; padding: 1px; text-align: left; vertical-align: $valign;">$info</td>
"""

auction_template="""
          $auction
"""

def html_card(cards):
    src = Template(card_template)
    all = {
        "spade" : cards["S"],
        "heart" : cards["H"],
        "diamond" : cards["D"],
        "club" : cards["C"]
    }
    return src.safe_substitute(all)

def html_board(vulnerable, dealer):
    arrows={"S": "&darr;","W": "&larr;","N": "&uarr;","E": "&rarr;"}
    src = Template(board_template)
    
    NONE_COLOR="#93C47D"
    VULNERABLE_COLOR="#E06666"

    all = {}
    if vulnerable == "NS":
        all["ns_vulnerable"]= VULNERABLE_COLOR
        all["ew_vulnerable"]= NONE_COLOR
    elif vulnerable == "EW":
        all["ew_vulnerable"]= VULNERABLE_COLOR
        all["ns_vulnerable"]= NONE_COLOR
    elif vulnerable == "NONE":
        all["ew_vulnerable"]= NONE_COLOR
        all["ns_vulnerable"]= NONE_COLOR
    else:
        all["ew_vulnerable"]= VULNERABLE_COLOR
        all["ns_vulnerable"]= VULNERABLE_COLOR
    all["dealer"] = arrows[dealer]
    
    return src.safe_substitute(all)

def bid_css(contract):
    rank = contract[0]
    suit = contract[1:]
    css = rank
    space="<span style='font: 10pt Times, serif'>&thinsp;</span>"
    suit_css = {
        'S':  space + "<span class=bcspades style='color:black;font: 10pt Verdana, sans-serif;'>&spades;</span>",
        "H":  space + "<span class=bchearts style='color: red; font: 10pt Verdana, sans-serif;'>&hearts;</span>",
        "D":  space + "<span class=bcdiams style='color: red; font: 10pt Verdana, sans-serif;'>&diams;</span>",
        "C":  space + "<span class=bcclubs style='color:black;font: 10pt Verdana, sans-serif;'>&clubs;</span>",
        "NT": space + "NT",
    }
    if contract in [ "AP", "Pass", "X", "XX" ]:
        css = contract
    else:
        css += suit_css[suit]
    return css

def contract_css(contract):
    rank = contract[0]
    suit = double = ""
    x = contract.find("X")
    if x == -1: # no x or xx
        suit = contract[1:]
    else:
        suit = contract[1:x]
        double = contract[x:]
    css = rank
    space="<span style='font: 10pt Times, serif'>&thinsp;</span>"
    suit_css = {
        'S':  space + "<span class=bcspades style='color:black;font: 10pt Verdana, sans-serif;'>&spades;</span>",
        "H":  space + "<span class=bchearts style='color: red; font: 10pt Verdana, sans-serif;'>&hearts;</span>",
        "D":  space + "<span class=bcdiams style='color: red; font: 10pt Verdana, sans-serif;'>&diams;</span>",
        "C":  space + "<span class=bcclubs style='color:black;font: 10pt Verdana, sans-serif;'>&clubs;</span>",
        "NT": space + "NT",
    }
    css += suit_css[suit] + double.lower()

    return css

def html_extra(contract, declarer,lead):
    src = Template(extra_template)
    css = contract_css(contract)
    # trans_declarer={"S": "南","W": "西","N": "北","E": "东"}
    # trans_declarer={"S": "S","W": "W","N": "N","E": "E"}
    suit_css = {
        'S':  "<span class=bcspades style='color:black;font: 10pt Verdana, sans-serif;'>&spades;</span>",
        "H":  "<span class=bchearts style='color: red; font: 10pt Verdana, sans-serif;'>&hearts;</span>",
        "D":  "<span class=bcdiams style='color: red; font: 10pt Verdana, sans-serif;'>&diams;</span>",
        "C":  "<span class=bcclubs style='color:black;font: 10pt Verdana, sans-serif;'>&clubs;</span>"
    }
    trans_declarer={"S": "South","W": "West","N": "North","E": "East"}
    all = { "declarer": trans_declarer[declarer], "contract" : css, "lead" : suit_css[lead[0]] + lead[1]}
    return src.safe_substitute(all)

def html_info(info, bottom=False):
    src = Template(info_template)
    info = info.replace("&","<br />")
    if bottom:
        valign = "bottom"
    else:
        valign = "top"
    all = { "info": info, "valign" : valign}
    return src.safe_substitute(all)

def html_auction(auction, section_auction):
    src = Template(auction_template)
    # need insert empty cell based on auction
    position="WNES" 
    empty_cells = position.index(auction)

    # handle ==1==, 6D?, 6D!
    filtered_auction = [x for x in section_auction.split() if not x.startswith("=")]
    # print(filtered_auction)
    css = "<tr class=bcauction>"
    col = 0
    for empty in range(empty_cells):
        css += "<td class=bcauction style='padding: 1px; text-align: left; white-space: nowrap'></td>\n"
        col += 1
    for one in filtered_auction:
        # print("aution:", one)
        one = one.replace("!", '').replace("?","")
        css +="<td class=bcauction style='padding: 1px; text-align: left; white-space: nowrap'>%s</td>\n" %  bid_css(one)
        if col == 3:
            col = 0
            css += "</tr>\n"
        else:
            col += 1
    css += "</tr>\n"
    all = { "auction": css}
    return src.safe_substitute(all)

def pbn2html(pbn_file):
    all = {
        "generated" : "2020-12-01"
    }

    # handle multiple pbn in one file
    pbnstr = open(pbn_file,encoding="utf-8" ).read()
    # delimiter="[Event"
    delimiter="\n*"
    if len(pbnstr.split(delimiter)) > 1:
        pbns =  [e for e in pbnstr.split(delimiter) if "Declarer" in e]
    else:
        delimiter="[Event" # old format used for larry
        pbns =  [delimiter+e for e in pbnstr.split(delimiter) if "Deal" in e]
    print("found", len(pbns), "boards")
    result = ""
    #print(pbns)
    for pbn in pbns:
        # print("==================================", pbn)
        try:
            tags, hands, section_auction = importPBN(pbn)
            #print(tags, hands,section_auction)
            all["title"] = tags["Event"]
            all["north"] = html_card(hands["N"])
            all["west"] = html_card(hands["W"])
            all["east"] = html_card(hands["E"])
            all["south"] = html_card(hands["S"])
            all["board"] = html_board(tags["Vulnerable"],tags["Dealer"])
            all["extra"] = html_extra(tags["Contract"],tags["Declarer"],tags["Lead"])
            all["auction"] = html_auction(tags["Auction"], section_auction)
            # hacked solution to check whether it is module or local
            # windows module
            # _: C:/Users/rdccaiy/AppData/Local/Programs/Python/Python38/Scripts/pbn2html
            # unix module
            # _: /home/rdccaiy/py3env/bin/pbn2html
            # local (python3.8)
            # _: ./pbn2html.py
            #for item, value in os.environ.items():
            #    print('{}: {}'.format(item, value))
            if "pbn2html.py" in os.environ.get("_"):
                template = open("template.html", "r",encoding="utf-8").read()
                src = Template(template)
            else:
                template = pkgutil.get_data(__name__,'template.html')
                src = Template(template.decode('utf-8'))

            result += src.safe_substitute(all)
        except ParseError as error:
            print(error)
        except KeyError as kerror:
            print("Key error: ", kerror)

        #print(len(result))
    src = Template(html_template)
    all = { "all": result, "version": version }
    result = src.safe_substitute(all)
    output = os.path.splitext(pbn_file)[0]+'.html'
    with io.open(output, "w", encoding="utf-8") as text_file:
        print("write to file %s" % output)
        text_file.write(result)

def get_from_pbn(pbn):
    tags, hands, section_auction = importPBN(pbn)
    pbn = {
        "tags": tags,
        "hands": hands,
        "section_auction" : section_auction
    }
    # print(hands)
    return pbn

def get_from_pbn_file(pbn_file):
    pbn = open(pbn_file,encoding="utf-8" ).read()
    return get_from_pbn(pbn)

def pbn_html_auction(pbn):
    all = {}
    tags = pbn["tags"]
    section_auction = pbn["section_auction"]
    all["auction"] = html_auction(tags["Auction"], section_auction)
    template = pkgutil.get_data(__name__,'auction_template.html')
    src = Template(template.decode('utf-8'))
    result = src.safe_substitute(all)
    return result

def pbn_html_deal(pbn, cards="NESW", ll="", ul="", ur=""):
    all = {}
    tags = pbn["tags"]
    hands = pbn["hands"]
    empty_hand = card_empty_template
    # print(hands["N"])

    all["north"] = html_card(hands["N"])
    all["west"] = html_card(hands["W"])
    all["east"] = html_card(hands["E"])
    all["south"] = html_card(hands["S"])
    for card in "NESW":
        if card not in cards:
            if card == "N":
                all["north"] = empty_hand
            if card == "W":
                all["west"] = empty_hand
            if card == "E":
                all["east"] = empty_hand
            if card == "S":
                all["south"] = empty_hand

    all["board"] = html_board(tags["Vulnerable"],tags["Dealer"])
    if ul == "":
        all["ul"] = html_extra(tags["Contract"],tags["Declarer"])
    else:
        all["ul"] = html_info(ul)
    all["ll"] = html_info(ll, bottom=True)
    all["ur"] = html_info(ur)

    template = pkgutil.get_data(__name__,'deal_template.html')
    src = Template(template.decode('utf-8'))
    #template = open("deal_template.html", "r", encoding="utf-8").read()
    #src = Template(template)
    result = src.safe_substitute(all)
    return result

def pbn_html_all(pbn):
    all = {}
    tags = pbn["tags"]
    hands = pbn["hands"]
    section_auction = pbn["section_auction"]
    all["north"] = html_card(hands["N"])
    all["west"] = html_card(hands["W"])
    all["east"] = html_card(hands["E"])
    all["south"] = html_card(hands["S"])
    all["board"] = html_board(tags["Vulnerable"],tags["Dealer"])
    all["extra"] = html_extra(tags["Contract"],tags["Declarer"])
    all["auction"] = html_auction(tags["Auction"], section_auction)
    template = pkgutil.get_data(__name__,'all_template.html')
    src = Template(template.decode('utf-8'))
    result = src.safe_substitute(all)
    return result

def main():
    # print(sys.argv)
    if len(sys.argv) > 1:
        param = sys.argv[1:]
        if param[0].endswith(".pbn"):
            pbn2html(param[0])
            pbn = get_from_pbn_file(param[0])
            #print(pbn_html_all(pbn))
            #print(pbn_html_auction(pbn))
            
            #print(pbn_html_deal(pbn, cards="NS"))
            #print(pbn_html_deal(pbn, cards="EW"))
            #print(pbn_html_deal(pbn, cards="NEWS"))
            #print(pbn_html_deal(pbn, ul=" ", ur="群组赛1209&牌号 4/8", ll="NS 4/12&EW 0"))
        else:
            print("pbn2html.py <file.pbn>")
    else:
        print("pbn2html.py <file.pbn>")

if __name__ == '__main__':
    main()

